@@ -18,13 +18,13 @@ pub enum ElementType {
     ISize,
     USize,
     String,
-    Guid,
+    GUID,
     IUnknown,
     IInspectable,
     HRESULT,
     TypeName,
     GenericParam(String),
-    Array((Box<Signature>, u32)), // TODO: store Blob rather than Signature or rip gen stuff out of Signature and use that?
+    Array((Box<Signature>, u32)),
     MethodDef(MethodDef),
     Field(Field),
     TypeDef(TypeDef),
@@ -43,6 +43,15 @@ impl From<TypeDef> for ElementType {
 }
 
 impl ElementType {
+    pub fn size(&self) -> usize {
+        match self {
+            Self::I64 | Self::U64 | Self::F64 => 2,
+            Self::GUID => 4,
+            Self::TypeDef(def) => def.size(),
+            _ => 1,
+        }
+    }
+
     pub fn row(&self) -> &Row {
         match self {
             Self::MethodDef(def) => &def.0,
@@ -91,7 +100,7 @@ impl ElementType {
             "isize" => Some(Self::ISize),
             "usize" => Some(Self::USize),
             "HSTRING" => Some(Self::String),
-            "Guid" => Some(Self::Guid),
+            "GUID" => Some(Self::GUID),
             "IInspectable" => Some(Self::IInspectable),
             _ => None,
         }
@@ -100,8 +109,6 @@ impl ElementType {
     pub fn type_name(&self) -> TypeName {
         match self {
             Self::TypeDef(def) => def.type_name(),
-            Self::MethodDef(def) => TypeName::new(def.parent().namespace(), def.name()),
-            Self::Field(def) => TypeName::new(def.parent().namespace(), def.name()),
             _ => unimplemented!(),
         }
     }
@@ -122,27 +129,51 @@ impl ElementType {
             Self::F64 => "f8".to_owned(),
             Self::String => "string".to_owned(),
             Self::IInspectable => "cinterface(IInspectable)".to_owned(),
-            Self::Guid => "g16".to_owned(),
+            Self::GUID => "g16".to_owned(),
             Self::TypeDef(t) => t.type_signature(),
             _ => unimplemented!(),
         }
     }
 
-    pub fn dependencies(&self, include: TypeInclude) -> Vec<TypeEntry> {
+    pub fn include_dependencies(&self, include: TypeInclude) {
         match self {
-            Self::MethodDef(t) => t.dependencies(),
-            Self::TypeDef(t) => t.dependencies(include),
-            Self::Field(t) => t.dependencies(include),
-            Self::Array((signature, _)) => signature.dependencies(include),
-            _ => Vec::new(),
+            Self::MethodDef(t) => t.include_dependencies(),
+            Self::TypeDef(t) => t.include_dependencies(include),
+            Self::Field(t) => t.include_dependencies(None, include),
+            Self::Array((signature, _)) => signature.kind.include_dependencies(include),
+            _ => {}
         }
     }
 
-    pub fn definition(&self, include: TypeInclude) -> Vec<TypeEntry> {
+    pub fn include_definition(&self, include: TypeInclude) {
         match self {
-            Self::TypeDef(t) => t.definition(include),
-            Self::Array((signature, _)) => signature.definition(include),
-            _ => Vec::new(),
+            Self::TypeDef(t) => t.include_definition(include),
+            Self::Array((signature, _)) => signature.kind.include_definition(include),
+            _ => {}
+        }
+    }
+
+    pub fn struct_features(
+        &self,
+        features: &mut BTreeSet<&'static str>,
+        keys: &mut std::collections::HashSet<Row>,
+    ) {
+        match self {
+            Self::TypeDef(def) => def.struct_features(features, keys),
+            Self::Array((signature, _)) => signature.kind.struct_features(features, keys),
+            _ => {}
+        }
+    }
+
+    pub fn method_features(
+        &self,
+        features: &mut BTreeSet<&'static str>,
+        keys: &mut std::collections::HashSet<Row>,
+    ) {
+        match self {
+            Self::TypeDef(def) => def.method_features(features, keys),
+            Self::Array((signature, _)) => signature.kind.method_features(features, keys),
+            _ => {}
         }
     }
 
@@ -168,7 +199,7 @@ impl ElementType {
             Self::TypeDef(t) => t.is_convertible(),
             Self::String
             | Self::IInspectable
-            | Self::Guid
+            | Self::GUID
             | Self::IUnknown
             | Self::GenericParam(_) => true,
             _ => false,
@@ -207,7 +238,7 @@ impl ElementType {
     pub fn is_udt(&self) -> bool {
         match self {
             Self::TypeDef(t) => t.is_udt(),
-            Self::Guid => true,
+            Self::GUID => true,
             _ => false,
         }
     }
