@@ -15,13 +15,39 @@ pub struct MethodParam {
 }
 
 impl MethodSignature {
-    pub fn dependencies(&self, include: TypeInclude) -> Vec<TypeEntry> {
+    pub fn include_dependencies(&self, include: TypeInclude) {
+        if let Some(return_sig) = &self.return_sig {
+            return_sig.kind.include_definition(include);
+        }
+
+        for param in &self.params {
+            param.signature.kind.include_definition(include);
+        }
+    }
+
+    pub fn method_features(&self) -> BTreeSet<&'static str> {
+        let mut features = std::collections::BTreeSet::new();
+        let mut keys = std::collections::HashSet::new();
         self.return_sig
             .iter()
-            .map(|s| s.definition(include))
-            .chain(self.params.iter().map(|p| p.signature.definition(include)))
-            .flatten()
-            .collect()
+            .for_each(|def| def.kind.method_features(&mut features, &mut keys));
+        self.params
+            .iter()
+            .for_each(|def| def.signature.kind.method_features(&mut features, &mut keys));
+        features
+    }
+
+    pub fn struct_features(
+        &self,
+        features: &mut BTreeSet<&'static str>,
+        keys: &mut std::collections::HashSet<Row>,
+    ) {
+        self.return_sig
+            .iter()
+            .for_each(|def| def.kind.struct_features(features, keys));
+        self.params
+            .iter()
+            .for_each(|def| def.signature.kind.struct_features(features, keys));
     }
 
     pub fn kind(&self) -> SignatureKind {
@@ -40,7 +66,7 @@ impl MethodSignature {
                         let guid = &self.params[self.params.len() - 2];
                         let object = &self.params[self.params.len() - 1];
 
-                        if guid.signature.kind == ElementType::Guid
+                        if guid.signature.kind == ElementType::GUID
                             && !guid.param.flags().output()
                             && object.signature.kind == ElementType::Void
                             && object.param.is_com_out_ptr()
@@ -77,6 +103,12 @@ impl MethodSignature {
 
         SignatureKind::PreserveSig
     }
+
+    pub fn size(&self) -> usize {
+        self.params
+            .iter()
+            .fold(0, |sum, param| sum + param.signature.size())
+    }
 }
 
 impl MethodParam {
